package com.softwareag.example.dirservice.crowd;

import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;

import com.softwareag.dir.extdirsvc.datamodel.IExternalDirDataService;
import com.softwareag.dir.extdirsvc.datamodel.IPrincipalModel;
import com.softwareag.example.dirservice.crowd.rest.CrowdRestConnect;
import com.softwareag.example.dirservice.crowd.rest.GroupRestProvider;
import com.softwareag.example.dirservice.crowd.rest.UserRestProvider;
import com.webmethods.common.search.query.ISearchQuery;
import com.webmethods.portal.PortalException;
import com.webmethods.portal.service.dir.DirectoryTools;
import com.webmethods.portal.service.dir.IDirSystem;
import com.webmethods.portal.service.view.IPagingCookie;
import com.webmethods.portal.system.IComponentData;
import com.webmethods.portal.system.impl.BaseComponent;
import com.webmethods.portal.system.init.InitializationException;

public class CrowdDirectoryService extends BaseComponent implements IExternalDirDataService {
	
	private static Logger log = Logger.getLogger(CrowdDirectoryService.class);
	
    private ConnectionConfig conConfig;
    
    private GroupRestProvider groupProvider = new GroupRestProvider();
    private UserRestProvider userProvider = new UserRestProvider();
	
	public CrowdDirectoryService() {
		conConfig = getDefaultConfig();
	}
	
	@Override
	public void init(IComponentData componentData, String phase) throws InitializationException {
		super.init(componentData, phase);
	}
	
	@Override
	public void shutdown() {
		super.shutdown();
	}
	
	@Override
	public boolean authenticateImpl(String uid, int type, Object credentials) {
		return userProvider.authenticate(CrowdRestConnect.getRootResource(conConfig), uid, credentials.toString());
	}

	@Override
	public List<IPrincipalModel> getMembership(int type, boolean parents, String dn, int resultSize) {
		String id = DirectoryTools.getUID(dn);		
		return internalGetMembership(type, parents, id, true, resultSize); 
	}
	
	/*
	 * Optional. 
	 * Valid only for nested groups.
	 * If the directory service can provide the nested parents of the group or user in a single call.
	 * If such shortcut method is not available, MWS will still return nested parent groups by iteration.
	 */
	@Override
	public List<IPrincipalModel> getMembershipShortCut(int type, String dn) {
		String id = DirectoryTools.getUID(dn);
		if (supportsMembershipShortCut()) {
			return internalGetMembership(type, true, id, false, -1);
		}
		return Collections.emptyList();
	}

	
	@Override
	public int getTotalPrincipalCount(int type) {
		return 333; // underlying source cannot provide this information (no API for this)
	}

	@Override
	public IPrincipalModel lookupByID(int type, String uid) {
		try {
			switch (type) {
				case IDirSystem.TYPE_USER:
					return userProvider.getUser(CrowdRestConnect.getRootResource(conConfig), uid);
				case IDirSystem.TYPE_GROUP:
					return groupProvider.getGroup(CrowdRestConnect.getRootResource(conConfig), uid);
			}
		} catch (Exception ex) {
			log.error("Could not find principal " + uid, ex);		
		}
		return null;
	}
	
	@Override
	public List<IPrincipalModel> listPrincipals(int type, int start, int end) {
		return listOrSearch(type, null, start, end-start);
	}

	@SuppressWarnings("deprecation")
	@Override
	public List<IPrincipalModel> search(int type, ISearchQuery query, IPagingCookie cookie) throws PortalException {
		try {
			return listOrSearch(type, query, cookie.getStart(), cookie.getPageSize());
		} catch (Exception ex) {
			throw new PortalException(ex.getMessage(), ex);
		}
	}
	
	@SuppressWarnings("unchecked")
	private List<IPrincipalModel> listOrSearch(int type, ISearchQuery query, int start, int maxResults) {
		try {
			switch (type) {
				case IDirSystem.TYPE_USER:
					return (List<IPrincipalModel>)(List<?>) 
							userProvider.searchUsers(CrowdRestConnect.getRootResource(conConfig), query, start, maxResults);
				case IDirSystem.TYPE_GROUP:
					return (List<IPrincipalModel>)(List<?>) 
							groupProvider.searchGroups(CrowdRestConnect.getRootResource(conConfig), query, start, maxResults);
			}
		} catch (Exception ex) {
			log.error("Failed searching for principals.", ex);
		}
		return Collections.emptyList();
	}
	

	@Override
	public void setConfigData(String data) {
		ConnectionConfig temp = new ConnectionConfig(data);
		if (temp.isValidConfig()) {
			conConfig = temp;
		} else {
			log.warn("Invalid external directory configuration passed, reverting to defaults");
			conConfig = getDefaultConfig();
		}
	}

	@Override
	public boolean supportsNestedGroups() {
		/*
		 * Configurable - default false. Also depends on the configuration of Crowd directory
		 */
		return conConfig.isSupportNestedGroups();
	}
	
	@Override
	public boolean supportsMembershipShortCut() {
		/*
		 * Configurable - default false. 
		 * Will result in usage of getMembershipShortCut method - see comments 
		 */
		return supportsNestedGroups() && conConfig.isUseNestedQueries();
	}
	
	
	@SuppressWarnings("unchecked")
	private List<IPrincipalModel> internalGetMembership(int type, boolean parents, String id, 
			boolean directMembers, int resultSize) {
		try {
			switch (type) {
				case IDirSystem.TYPE_USER:
					return (List<IPrincipalModel>)(List<?>) 
							userProvider.getMembership(CrowdRestConnect.getRootResource(conConfig), id, directMembers, resultSize);
				case IDirSystem.TYPE_GROUP:
					return (List<IPrincipalModel>)(List<?>) 
							groupProvider.getMembership(CrowdRestConnect.getRootResource(conConfig), parents, id,
									conConfig.isSupportNestedGroups(), directMembers, resultSize);
			}
		} catch (Exception ex) {
			log.error("Failed getting membership", ex);
		}
		return Collections.emptyList();
	}
		
	private ConnectionConfig getDefaultConfig() {
		ConnectionConfig cfg = new ConnectionConfig();
		cfg.setUsername("username");
		cfg.setPassword("password");
		cfg.setConnectionUrl("http://localhost:8095/crowd");
		return cfg;
	}

}
